# cellpot.py
import os
import numpy as np
import scipy.integrate
import math
import matplotlib.pyplot as plt

def cellpot():
    ndim = 6
    ttime = 600.0
    dt = 0.05
    npts = int(ttime/dt)+1
    istate = 0

    Dm = 60.0     # (micrometer) diameter of the cell
    Lm = 70.0     # (micrometer) length of the cell

    Eleak =  -65.0 # (mV)
    ENa   =   50.0 # (mV)
    EK    = -100.0 # (mV)
    ECa   =   50.0 # (mV)

    dGleak = 0.002 # (mS/cm^2)
    dGNa   = 0.05  # 0.53;   // (mS/cm^2)
    dGK    = 0.15  # 0.13;  // (mS/cm^2)
    dGTCa  = 0.03  # (mS/cm^2)

    vtraub = -55.0 # (mV)

    Iinj = -2000.0 # 2000.0;  //(pA)
    Ton  = 50.0
    Toff = 300.0   # 40;
    vinit = -63.27

    fac = 10     # conversion factor (mS/cm^2) -> (pS/micrometer^2)

    Am = 3.14 * Dm * Lm # (squre_micrometer) surface area
    Cm = 0.01 * Am
    InvCm = 1/Cm

    Gleak = Am * dGleak * fac
    GNa   = Am * dGNa * fac
    GK    = Am * dGK * fac
    GTCa  = Am * dGTCa * fac

    def get_Na_m(v):
        v2 = v-vtraub
        a = 0.32 * (13-v2) / ( math.exp((13-v2)/4) - 1)
        b = 0.28 * (v2-40) / ( math.exp((v2-40)/5) - 1)
        return [a,b]

    def get_Na_h(v):
        v2 = v-vtraub
        a = 0.128 * math.exp((17-v2)/18)
        b = 4 / ( 1 + math.exp((40-v2)/5) )
        return [a,b]

    def get_K_n(v):
        v2 = v - vtraub
        a = 0.032 * (15-v2) / ( math.exp((15-v2)/5) - 1)
        b = 0.5 * math.exp((10-v2)/40)
        return [a,b]

    def get_TCa_m(v):
        vshift=0
        vs = v + vshift
        tau = ( 3 + 1.0 / ( math.exp((vs+25)/10) +  math.exp(-(vs+100)/15)))
        inf = 1.0 / ( 1 + math.exp(-(vs+50)/7.4) )
        return [tau,inf]

    def get_TCa_h(v):
        vshift=0
        vs = v + vshift
        tau = (85.0+1.0/(math.exp((vs+46.0)/4.0) + math.exp(-(vs+405.0)/50.0) ) )
        inf = 1.0 / ( 1.0 + math.exp((vs+78.0)/5.0) )
        return [tau, inf]

    def setInitialValues(y):
        y[0] = vinit
        v = y[0]
        [a,b] = get_Na_m(v)
        y[1] = a/(a+b)

        [a,b] = get_Na_h(v)
        y[2] = a/(a+b)

        [a,b] = get_K_n(v)
        y[3] = a/(a+b)

        [tau,inf] = get_TCa_m(v)
        y[4] = inf

        [tau,inf] = get_TCa_h(v)
        y[5] = inf


    def rhs(y,t):  # return derivatives of the array y
        if t >= Ton and  t < Toff :
            c = Iinj
        else:
            c = 0.0

        v = y[0];
        INa = GNa * y[1]**3 * y[2]  * (v-ENa)
        IK  = GK  * y[3]**4 * (v-EK)
        ITCa = GTCa * pow(y[4],2.0) * y[5] * (v - ECa)

        [a1,b1]=get_Na_m(v)
        [a2,b2] = get_Na_h(v)
        [a3,b3] = get_K_n(v)
        [tau1,inf1] = get_TCa_m(v)
        [tau2,inf2] = get_TCa_h(v)

        return np.array(
          [ InvCm * (c - Gleak * (v-Eleak) - INa - IK -  ITCa ),
            a1 - (a1+b1)*y[1],
            a2 - (a2+b2)*y[2],
            a3 - (a3+b3)*y[3],
            (inf1 - y[4]) / tau1,
            (inf2 - y[5]) / tau2])

# main
    t = np.linspace(0.0, ttime, npts)
    y0 = np.zeros(ndim)
    setInitialValues(y0)
    y = scipy.integrate.odeint(rhs, y0, t)

    th = -20.0
    sn = 0
    for i in range(npts-1):
        if y[i+1,0] >= th and y[i,0] < th:
           sn = sn+1
    print('number of spikes = ', sn)

    plt.figure()
    plt.plot(t, y[:,0])
    plt.xlabel('time (ms)')
    plt.ylabel('voltage (mV)')
    plt.show()

if __name__=='__main__':
    cellpot()
